import os
import sys
import math
import numpy
import arcpy
from arcpy.sa import *

class Toolbox(object):
    def __init__(self):
        """Define the toolbox (the name of the toolbox is the name of the
        .pyt file)."""
        self.label = "Storage Capacity v1.0"
        self.alias = "storage"

        # List of tool classes associated with this toolbox
        self.tools = [StorageCapacity]


class StorageCapacity(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Storage Capacity"
        self.description = "Calculate storage capacities below a given series of elevation values"
        self.canRunInBackground = False

    def getParameterInfo(self):
        """Define parameter definitions"""
        param0 = arcpy.Parameter(
            displayName="Input DEM",
            name="in_dem",
            datatype="GPRasterLayer",
            parameterType="Required",
            direction="Input")
        param1 = arcpy.Parameter(
            displayName="Output Table",
            name="out_table",
            datatype="DETable",
            parameterType="Required",
            direction="Output")
        param2 = arcpy.Parameter(
            displayName="Input boundary polygon",
            name="in_polygon",
            datatype="GPFeatureLayer",
            parameterType="Optional",
            direction="Input")
        param3 = arcpy.Parameter(
            displayName="Maximum elevation",
            name="max_elevation",
            datatype="GPDouble",
            parameterType="Optional",
            direction="Input")
        param4 = arcpy.Parameter(
            displayName="Minimum elevation",
            name="min_elevation",
            datatype="GPDouble",
            parameterType="Optional",
            direction="Input")
        param5 = arcpy.Parameter(
            displayName="Incremental",
            name="incremental",
            datatype="GPDouble",
            parameterType="Optional",
            direction="Input")
        params = [param0, param1, param2, param3, param4, param5]
        return params

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""
        if parameters[0].altered == True and parameters[3].altered == False and parameters[4].altered == False and parameters[5].altered == False:
            try:
                rasobj1 = arcpy.sa.Raster(parameters[0].valueAsText)
                max1 = rasobj1.maximum
                min1 = rasobj1.minimum
                parameters[3].value = max1
                parameters[4].value = min1
                parameters[5].value = (max1 - min1) / 10.0
            except:
                pass
        return


    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        arcpy.CheckOutExtension("spatial")
        arcpy.env.overwriteOutput = True
        #arcpy.env.scratchWorkspace = "in_memory"

        in_dem = parameters[0].valueAsText
        out_table = parameters[1].valueAsText
        in_polygon = parameters[2].valueAsText
        in_max = parameters[3].value
        in_min = parameters[4].value
        in_increm = parameters[5].value

        #clip raster
        if in_polygon <> None:
            clip_output = arcpy.env.scratchWorkspace + os.sep + "clxgwttt_"
            arcpy.Clip_management(in_raster=in_dem,
                                  out_raster=clip_output,
                                  in_template_dataset=in_polygon,
                                  clipping_geometry="ClippingGeometry",
                                  maintain_clipping_extent="NO_MAINTAIN_EXTENT")
        else: # no clip
            clip_output = in_dem

        in_rasobj1 = Raster(clip_output)
        min1 = in_rasobj1.minimum
        max1 = in_rasobj1.maximum
        cx = in_rasobj1.meanCellWidth
        cy = in_rasobj1.meanCellHeight
        carea = cx*cy

        incre_factor = int(math.ceil((in_max - in_min) / in_increm))
        result_list = []
        for i in xrange(incre_factor):
            lower1 = in_min + in_increm * i
            upper1 = in_min + in_increm * (i + 1)
            newval1 = upper1
            out_con = arcpy.env.scratchWorkspace + os.sep + "conxgwttt_"
            result_con = Con(Raster(clip_output) < upper1, clip_output)
            out_minus = Minus(upper1, result_con)
            in_zone = Int(result_con * 0) + 1
            out_zonal_sum = arcpy.env.scratchWorkspace + os.sep + "znxgwsttt_"

            try:
                arcpy.gp.ZonalStatisticsAsTable_sa(in_zone, "Value", out_minus, out_zonal_sum, "DATA", "SUM")
                with arcpy.da.SearchCursor(out_zonal_sum, ("SUM")) as cur:
                    for row in cur:
                        sum_diff = row[0]
                sum_vol = sum_diff * carea
                temp_count = arcpy.env.scratchWorkspace + os.sep + "cntxgwt_"
                arcpy.CopyRows_management(in_zone, temp_count)
                with arcpy.da.SearchCursor(temp_count, ("Count")) as cur:
                    for row in cur:
                        sum_count = row[0]
                sum_area = sum_count * carea
            except:
                sum_vol = None
                sum_area = None
            result_list.append((upper1, sum_area, sum_vol))

        dt = numpy.dtype([('Elevation', '<f8'), ('Area', '<f8'), ('Volume', '<f8')])
        out_np = numpy.array(result_list, dtype = dt)
        arcpy.da.NumPyArrayToTable(out_np, out_table)
